@@ -1,11 +1,10 @@
-# $Id: /mirror/gungho/lib/Gungho/Provider.pm 31105 2007-11-26T06:18:38.487252Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Provider.pm 6471 2007-04-11T23:53:04.413297Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
 
 package Gungho::Provider;
 use strict;
-use warnings;
 use base qw(Gungho::Base);
 use Gungho::Request;
 
@@ -16,13 +15,20 @@ sub dispatch {}
 sub dispatch_request
 {
     my ($self, $c, $req) = @_;
-    $c->send_request($req);
+    eval {
+        $c->send_request($req);
+    };
+    if (my $e = $@) {
+        if ($e->isa('Gungho::Exception::RequestThrottled')) {
+            # This request was throttled. Attempt to do it later
+            $self->pushback_request($c, $req);
+        } else {
+            die $e;
+        }
+    }
 }
 
-sub pushback_request {
-    my ($self, $c, $req) = @_;
-    die ref($self) . '::pushback_request is not implemented';
-}
+sub pushback_request {}
 
 1;
 
