@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Request.pm 6472 2007-04-11T23:57:08.645886Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Request.pm 7031 2007-05-05T22:50:16.562488Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -27,6 +27,14 @@ sub _find_digest_class
     };
 }
 
+sub new
+{
+    my $class = shift;
+    my $self  = $class->SUPER::new(@_);
+    $self->{_notes} = {};
+    return $self;
+}
+
 sub id
 {
     my $self = shift;
